.module/ram/abs=0      C1;
.pagelength  59;                    { Works for .LST files with 550C }
.include    <system.k>;

{************************************************************************
 *
 *      EZ-KIT Lite DSP Program for SOSSD
 *	c1sin.dsp  -  Generate a single 1000 Hz sine wave
 *	24 March 1999    RSL
 *
 ************************************************************************}

{********************   PROGRAM MEMORY     *******************************}
{ The following are polynomial coefficients used in the calculation of
  sin(x) for x between 0 and 90 deg.  Tables of constant numbers are most
  easily used when they are in program memory.
  Program memory has 24 bit words.  When entering a data table it is
  necessary to right fill 8 bits with zeros.  In hexadecimal, this
  results in 6 hex digits with the last two always being 0's.  See the sin
  coefficient table below.             }
.var/pm         sin_coeff[5];
.init           sin_coeff: H#324000, H#005300, H#AACC00, H#08B700, H#1CCE00;

{********************   DATA MEMORY     *******************************}
{ SPORT0, i.e., Serial PORT 0, is located in data memory.
  The AD1847 Codec hardware operates independently of the DSP.
  But when A/D conversion is complete there is an interrupt generated
  by SPORT0.  This occurs 48,000 times per second as determined
  by the crystal clock in the Codec.  This interrupt causes, among other
  things, a jump to location 14 Hex in program memory.  This, in turn,
  is sent off to process the newly received data, or to do any other task
  required on a periodic basis, such as generation of waveforms.
  It is at this time that new data can be sent to the Codec to be converted
  by the D/A in the Codec.  Memory locations tx_buf and rx_buf provide the
  interaction with the SPORT0 (and thus the Codec) when interrupts occur. }
.var/dm/circ	tx_buf[3];		{ SPORT0: Cmd + L data + R data }
.var/dm/circ 	rx_buf[3];		{ SPORT0: Status + L data + R data }

{ Initialization of the AD1847 Codec is complex.  The internal Codec
  registers are transfered out of the following buffer.  It is circular to
  allow detection of a complete load of commands.     }
.var/dm/circ    init_cmds[13];		{ Codec Initialization }
.var/dm		stat_flag;		{ Codec Init status }

.var/dm         dphase;			{ For generation of sine wave }
.var/dm		phase;			{ This is cumulated phase }

{********************  Interrupt vector table  *************************}
	{ A jump to location 0 occurs when the dsp is reset }
        jump start;  rti; rti; rti;     {00: reset }

	{ Most of the interrupt vectors are not used, such as the next 3 }
        rti;         rti; rti; rti;     {04: IRQ2 }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }

	{ Do to the variable stat_flag, this interrupt routine is only used
	  during initialization of the AD1847 Codec.  It is used to send
	  register data to the Codec.  By using the interrupt the data is
	  not sent when the SPORT0 transmit buffer is full.         }
	ar = dm(stat_flag);		{10: SPORT0 tx }
        ar = pass ar;			{  see if needs initialization }
        if eq rti;			{ If dm(stat_flag)==0 exit this int }
        jump next_cmd;			{   else go transfer a control word }

	{ The following instructions are executed when an A/D conversion
	  cycle has been completed in the AD1847 Codec, and the resulting data
	  has been serially transmitted to the SPORT0.  The SPORT0 hardware
	  then generates an interrupt with an associated jump to this
	  program address: }
	jump input_samples;             {14: SPORT0 rx }
                     rti; rti; rti;

	{ And more unused interrupt locations: }
        rti;         rti; rti; rti;     {18: IRQE }
        rti;         rti; rti; rti;     {1c: BDMA }
	rti;         rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
	rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }

{*****************************  START  ***************************************
 * Initialize process and drop into main loop.  None of the index registers
 * have been initialized, so we need to turn off all interrupts before
 * the hardware is allowed to generate one.  Then we are safe with instructions
 * like dm(i0, m0)=ar.  This is the dreaded uninitialized pointer problem!!
 ****************************************************************************}
start:	imask=0;			{ Turn off all interrupts }

	{ Initialization has been broken into two pieces to simplify testing.
	  Most items should go into init0, but those that involve interrupts
	  can be put into init1 so that they can be commented out easily
	  when using the simulator.  }
	call init0;			{ Instructions that simulate easily }

	call init1;			{ And those that do not }

{****************     Main Program Loop    ******************************
 *  Main processing loop.  Does all non-periodic (background) processing.
 *  This loop is interrupted at a 48KHz rate to send and receive Codec data.
 *************************************************************************}
again:
	{ We have no background processing.  If we did, it would go here. }

	jump again;			{ Go round and round forever }

{***********************************************************************
 *                 SPORT0 Interrupt Handler                            *
 *  SPORT0 inputs from the Codec put us here 48000 times per second.   *
 *  Routines here are those that need this periodic service.           *
 ***********************************************************************}
input_samples:

	{ We do not know where calculations were at when the interrupt was
	  executed.  All of the processor registers that are used by the
	  background process must be protected, if they are to be used
	  by the interrupting routine, where we are now.  This is done by
	  having two sets of most registers and switching them with
	  the "ena sec_reg" instruction.  Note that the i index registers,
	  and the associated m and l registers are not switched.  There
	  is only one set of these 24 registers.   }
        ena sec_reg;                    { use secondary register bank }

	{ The following is the direct synthesis generator of sine waves. A
	  phase increment is added to the last value of phase.  This new
	  phase value is stored for use 20.83 microseconds later, and 
	  also given as an input to the sin routine.  A sine wave is
	  then calculated, ready to go to the D/A.  Note that the most
	  negative number (8000hex or -32768d is -180 degrees and the most
	  positive number, 7FFFhex or 32767d is +180 degrees.  Two's
	  complement arithmetic does the phase rollover for no cost.
	  That can be seen by observing that 7FFFhex + 1 = 8000hex.
	  For a 48 kHz sample rate this can be used up to 24 kHz as a
	  mathematical representation, and when run through the AD1847
	  D/A it is useful to about 22 kHz.    }
	ax1 = dm(dphase);		{ Phase increment for oscillator }
	ay1 = dm(phase);		{ Last phase }
	ar = ax1 + ay1;			{ New phase }
	ax0 = ar;			{ The phase input to sin is reg ax0 }
	dm(phase) = ar;			{ Save for next data point }
	call sin;			{ Phase in ax0, Sin returned in ar }

	dm(tx_buf+1) = ar;		{ Send sine wave to Left D/A (Codec) }
	dm(tx_buf+2) = ar;		{ Right D/A }

	dis sec_reg;			{ Restore the primary registers }
	rti;				{ This undoes the interrupt }

{*************************************************************************
 *
 * Sine waves are calculated by a polynomial approximation.  The
 * calculation is done only for 0 to 90 degrees and then the symmetry
 * of the sine wave is used to find the other three quadrants.
 * The phase change from point to point is 1.365333 times the desired
 * frequency in Hz.  The phase change must be rounded to the nearest
 * integer.
 * Note that the coefficients have been divided by 8 relative to 1.15 format.
 * That is to say, they are in 4.11 format.  This prevents overflow of
 * intermediate results.  The final result is then multiplied by 8
 * to bring the output to 1.15 format.
 *
 *       Sine Approximation:     ar = Sin(ax0)
 *       Calling Parameters:     ax0 in scaled 1.15 format; -1.0 is -180 deg
 *				    and 1.0 corresponds to 180 degrees
 *       Return Values:          ar in 1.15 format
 *       Computation Time:       25 cycles
 * Thanks to Analog Devices Application Handbook #1 for the basis of this
 * function.
 *************************************************************************}
sin:    m4=1;    l4=0;			{ Use i4,m4 index registers to }
        i4=^sin_coeff;                  {   point to polynomial coeffs }
        ay0=H#4000;			{ This is 90 degrees }
        ar=ax0, af=ax0 and ay0;         { Check 2nd or 4th quad. }
        if ne ar=-ax0;                  { If yes, negate input }
        ay0=H#7FFF;			{ This is a mask to replicate data, }
        ar=ar and ay0;                  {   but to remove the sign bit }
        my1=ar;
        mf=ar*my1 (RND), mx1=pm(i4,m4); { mf = input**2 }
        mr=mx1*my1 (SS), mx1=pm(i4,m4); { Start polynomial calculation }
        cntr=3;				{ Loop for 3 of 5 coefficients }
        do approx until ce;
                mr=mr+mx1*mf (ss);	{ More polynomial calculation }
approx:         mf=ar*mf (rnd), mx1=pm(i4,m4); { Power increase; get next coef}
        mr=mr+mx1*mf (ss);		{ Do last polynomial calculation }
        sr=ashift mr1 by 3 (hi);	{ Mult *8 (shift left 3) }
        sr=sr or lshift mr0 by 3 (lo);  { Convert to 1.15 format }
        ar=pass sr1;			{ See if result >=1.0 }
        if lt ar=pass ay0;              { If so, saturate, i.e. set to 0x7FFF }
        af=pass ax0;			{ See if input was negative }
        if lt ar=-ar;                   { If so, negate output}
        rts;

{******************************************************************************
 * Main startup initialization.  Many variables and registers require
 * startup initialization, either for operation or to improve simulation.
 * This routine includes the parts that can easily be simulated
 *****************************************************************************}
init0:	{ If there were very many of them, it would be desirable to collect
	  all the constants, such as 1365, at the beginning of the program
	  so that they could be easily found and changed.  It also makes the
	  program easier to read, and if the constant is used more than once,
	  it will always be the same!  See c1spn.dsp for an example.   }
	ax0=1365;			{ 1000.24 Hz }
	dm(dphase)=ax0;

	{ i7, l7, i6, l6 devoted to auto-buffering in SPORT0.  Initialized
	  here. These need protection, if used elsewhere in the program.    }
        i7 = ^rx_buf;
        l7 = %rx_buf;
        m7 = 1;
        i6 = ^tx_buf;
        l6 = %tx_buf;
        m6 = 1;
	{ i5, l5, m5 used in next_cmd to load AD1847 regs and then made free. }
        i5 = ^init_cmds;
        l5 = %init_cmds;
        m5 = 1;

        ax0=0xC000; dm(tx_buf)=ax0;    { Mode Change Enable (MCE)  initially }
        ax0=0x0000; dm(tx_buf+1)=ax0;
        ax0=0x0000; dm(tx_buf+2)=ax0;

	{  AD1847  Command Register Initialization.  They are placed into
	   memory here and will be loaded into the AD1847 with interrupt
	   routines in init1.  The details of these registers are given
	   in the AD1847 data sheet.  Note that the destinations are given
	   by the second hex digit and that register addresses b, e and f
	   are not valid, or used.    }
        ax0=0xc002;  dm(init_cmds)=ax0;	   { Left Input control }
        ax0=0xc102;  dm(init_cmds+1)=ax0;  { Right Input control }
        ax0=0xc288;  dm(init_cmds+2)=ax0;  { Left Aux #1 control }
        ax0=0xc388;  dm(init_cmds+3)=ax0;  { Right Aux #1 control }
        ax0=0xc488;  dm(init_cmds+4)=ax0;  { Left Aux #2 control }
        ax0=0xc588;  dm(init_cmds+5)=ax0;  { Right Aux #2 control }
        ax0=0xc680;  dm(init_cmds+6)=ax0;  { Left DAC control }
        ax0=0xc780;  dm(init_cmds+7)=ax0;  { Right DAC control }
        ax0=0xc85c;  dm(init_cmds+8)=ax0;  { Format: 48 KHz, (c850h for 8 KHz)}
        ax0=0xc909;  dm(init_cmds+9)=ax0;  { Interface configuration }
        ax0=0xca00;  dm(init_cmds+10)=ax0; { Pin configuration }
        ax0=0xcc40;  dm(init_cmds+11)=ax0; { Misc Information }
        ax0=0xcd00;  dm(init_cmds+12)=ax0; { Digital Mix control }

{********************* Serial PORT 0 Initialization *************************
 * The following sets up the SPORT0 for autobuffering of 3 receive words
 * (Status, L data, and R data) or 3 transmit words (Command, L data, and
 * R data.)  These are data memory locations, like dm(SPORT0_Autobuf).
 * The actual locations in memory come from the system.k file and the details
 * of the data loaded to these dm locations is given in the Analog Devices
 * "ADSP-2100 Family Users Manual."
 *****************************************************************************}
        ax0= b#0000110101111111;   dm (SPORT0_Autobuf) = ax0;
        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;

{******************* System and memory initialization ***********************
 * A variety of hardware dependent parameters are are set by the following
 * registers.  The first two are in data memory and ifc, icntl and mstst are
 * three non-memory registers that control hardware operation.
 * The actual locations in memory come from the system.k file, whereas the
 * three non-memory registers are understood directly by the assembler.
 * The details of the data loaded to these dm locations is given in the
 * Analog Devices "ADSP-2100 Family Users Manual."
 *****************************************************************************}

	ax0 = 0;			{ No wait states are used }
	dm(DM_Wait_Reg)=ax0;
        ax0 = b#0001000000000000;	{ Enable SPORT0 }
        dm(System_Control_Reg) = ax0;
        ifc = b#0000000011111111;	{ Clear any pending interrupt }
        nop;
	icntl = 0;			{ Interrupts level sensitive, no nest }
	mstat = b#1000000;		{ Primary data registers }
	rts;

{*****************************************************************************
 * Start up initialization of the AD1847 Codec.  This requires some effort to
 * simulate, and it is easiest to skip this part when running the sim21
 * simulator.  This method doesn't support all testing, but is OK for most
 * purposes.
 * This part of the program is the "background" routine during the
 * initialization of the AD1847 Codec.  The SPORT0 Tx interrupt is enabled.
 * The transmission of data to the Codec will program the
 * Codec control registers.  See the AD1847 data sheet for the myriads of
 * details involved.
 *****************************************************************************}
init1:	{ AD 1847 Codec Initialization
	  The stat_flag=0 will prevent reinit of AD1847 from SPORT0 TX ints }
	ax0 = 1;			{ Set flag so control words will be }
	dm(stat_flag) = ax0;		{   be loaded by next_cmd routine }
	imask = b#0001000000;		{ Enable SPORT0 Tx int, disable others}
	ax0 = dm(i6, m6);		{ Start interrupt }
	tx0 = ax0;
check_init:
        ax0 = dm (stat_flag);		{ Wait for entire initialization }
        af = pass ax0;			{   command buffer (init_cmds) to be }
        if ne jump check_init;		{   sent to the codec by next_cmd   }
        ay0 = 2;
check_acih:
        ax0 = dm (rx_buf);		{ Once initialized, wait for codec }
        ar = ax0 and ay0;		{   to come out of autocalibration }
        if eq jump check_acih;		{ Wait for bit set }
check_aci1:
        ax0 = dm (rx_buf);		{ Wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci1;
	idle;				{ Sit and wait for an interrupt }

        ay0 = 0xbf3f;			{ Unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);	{ Unmute right DAC }
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

	ax0 = 0xc901;			{ Clear autocalibration request }
	dm(tx_buf)=ax0;
	idle;

        ax0 = 0x8000;			{ Clear over-range flags }
        dm (tx_buf) = ax0;
        idle;

        ifc = b#00000011111111;		{ Clear any pending interrupt }
        nop;
        imask = b#0000100000;		{ Enable RX0 interrupt, now our only }
	rts;

{************************  NEXT_CMD  ****************************************
 * Transmit Interrupt Service Routine used for Codec initialization
 * DAG 5 registers used here, but are available after this routine is run.
 * The commands are loaded here into the command location dm(tx_buf+0) where
 * they will be transferred to the Codec.
 * Adapted from EZ-KIT Lite routines.
 ****************************************************************************}
next_cmd:
        ena sec_reg;			{ Get other set of registers }
        ax0 = dm (i5, m5);		{ Get next control word and }
        dm (tx_buf+0) = ax0;		{   place in command word }
        ax0 = i5;			{ Now see if finished by comparing }
        ay0 = ^init_cmds;		{   the i5 address with the starting}
        ar = ax0 - ay0;			{   i5 value. Circ buff brings here}
        if gt rti;                  	{ Exit if more control words waiting }
        ax0 = 0x8000;			{ else clear the Mode Change Enable }
        dm (tx_buf+0) = ax0;		{   (MCE) bit in the Codec cntrl word}
        ax0 = 0;			{ Flag the completion by clearing }
        dm (stat_flag) = ax0;		{   stat_flag, stopping calls to here }
        rti;

.endmod;
